//---------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DcjBtnEdt.h"
#pragma package(smart_init)
//---------------------------------------------------------------------
#pragma resource "*.dfm"
TDcjButtonsEditDlg *DcjButtonsEditDlg;
//---------------------------------------------------------------------------
namespace Dcjbtnedt
{
  void __fastcall PACKAGE Register()
  {
    // register component editor and property editor
    RegisterPropertyEditor(__typeinfo(TDcjButtons), NULL, "",
        __classid(TDcjButtonsProperty));
    RegisterComponentEditor(__classid(TDcjMessageDlg),
        __classid(TDcjButtonsEditor));
  }
}
//---------------------------------------------------------------------------
// Component Editor Implementation
//---------------------------------------------------------------------------
__fastcall TDcjButtonsEditor::TDcjButtonsEditor(Classes::TComponent* AComponent,
  		TFormDesigner* ADesigner)
	: Dsgnintf::TDefaultEditor(AComponent, ADesigner)
{
}
//---------------------------------------------------------------------------
int __fastcall TDcjButtonsEditor::GetVerbCount(void)
{
  // return the number of new menu items to display
  return 2;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDcjButtonsEditor::GetVerb(int Index)
{
AnsiString result;

  // get the menu item captions
  switch( Index ) {
    case 0:
      result = "Preview";
      break;
    case 1:
      result = "Edit Buttons...";
  }
  // return the result
  return result;
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditor::ExecuteVerb(int Index)
{
  // execute the function for the selected menu item
  switch( Index ) {
    case 0:
      // preview the message dialog
      Edit();
      break;
    case 1:
      // display the buttons property editor
      EditButtons();
      break;
  }
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditor::Edit(void)
{
  // preview the message dialog
  TDcjMessageDlg *dialog = dynamic_cast<TDcjMessageDlg *>(Component);
  dialog->Execute();
  Application->MessageBox(("DcjMessageDlg returned the following results:\r\n\r\n"
      "    ModalResult = " + IntToStr(dialog->ModalResult) + "\r\n"
      "    ButtonID = " + IntToStr(dialog->ButtonID)).c_str(),
      "DcjMessageDlg Results",
      MB_ICONINFORMATION | MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditor::EditButtons(void)
{
  // display the buttons property editor
  TDcjButtonsEditDlg *BtnDlg = new TDcjButtonsEditDlg(Application);
  try {
    // set the buttons editor settings
    BtnDlg->Caption = Component->Name + " - " + BtnDlg->Caption;
    // copy the component buttons to the button editor dialog
    BtnDlg->Buttons->Assign(dynamic_cast<TDcjMessageDlg *>(Component)->Buttons);

    // display the buttons in the listbox
    BtnDlg->DisplayButtons();

    // display the dialog modally and check the result
    if( BtnDlg->ShowModal() == mrOk ) {
      // assign the modified buttons to the component Buttons property
      dynamic_cast<TDcjMessageDlg *>(
          Component)->Buttons->Assign(BtnDlg->Buttons);
      // let BCB know that we made the change
      Designer->Modified();
    }
  }
  catch(...) {
    Application->MessageBox("Error while loading property editor.",
        "Property Editor", MB_OK);
    if( BtnDlg ) delete BtnDlg;
  }
  delete BtnDlg;
}

//---------------------------------------------------------------------------
// Property Editor Implementation
//---------------------------------------------------------------------------
__fastcall TDcjButtonsProperty::TDcjButtonsProperty(void)
  : TPropertyEditor()
{
}
//---------------------------------------------------------------------------
__fastcall TDcjButtonsProperty::~TDcjButtonsProperty(void)
{
}
//---------------------------------------------------------------------------
TPropertyAttributes __fastcall TDcjButtonsProperty::GetAttributes(void)
{
  // enable the ellipses so we can display the dialog
  return (TPropertyAttributes() << paDialog);
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsProperty::Edit(void)
{
  // use auto_ptr so that the object is destroyed automatically
  // on exit of the function.
  TDcjButtonsEditDlg *BtnDlg = new TDcjButtonsEditDlg(Application);
  try {
    // set the buttons editor settings
    // note: in BCB3, GetComponent returns a pointer to a TPersistent object,
    // but under BCB1 it returns a pointer to a TComponent object.  go figure.
    // I don't care for those ridiculous long captions anyway.  We'll take the
    // easy way out.
    BtnDlg->Caption = GetName() + " - " + BtnDlg->Caption;
    // copy the buttons to the dialog editor.  note: we need to use
    // reinterpret_cast to let the compiler know we are converting from
    // a long (memory address) integral type to a pointer to a TDcjButtons
    // object.
    TDcjButtons *btns = reinterpret_cast<TDcjButtons *>(GetOrdValue());
    BtnDlg->Buttons->Assign(btns);

    BtnDlg->DisplayButtons();

    // display the dialog modally and check the result
    if( BtnDlg->ShowModal() == mrOk ) {
      btns->Assign(BtnDlg->Buttons);
      // set the properties.  again we use reinterpret_cast because we are
      // converting from a pointer to an integral type.
      SetOrdValue( reinterpret_cast<long>(btns) );
      Designer->Modified();
    }
  }
  catch(...) {
    Application->MessageBox("Error while loading property editor.",
        "Property Editor", MB_OK);
    if( BtnDlg ) delete BtnDlg;
  }
  delete BtnDlg;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDcjButtonsProperty::GetValue(void)
{
  // return the display string for the OI
  TVarRec v[] = { GetPropType()->Name };
  return Format("(%s)", v, ARRAYSIZE(v) - 1);
}

//---------------------------------------------------------------------------
// TDcjButtonsEditDlg Implementation
//---------------------------------------------------------------------------
__fastcall TDcjButtonsEditDlg::TDcjButtonsEditDlg(TComponent* AOwner)
	: TForm(AOwner)
{
  // construct the TDcjButtons object
  FButtons = new TDcjButtons(AOwner);

  iCurrentIndex = -1;

  // fill the combo box with the modal result strings
  for(int i = 0; i < 8; i++)
    cboModalResult->Items->Add(MODAL_RESULT[i]);

}
//---------------------------------------------------------------------------
__fastcall TDcjButtonsEditDlg::~TDcjButtonsEditDlg(void)
{
  // free the FButtons. note: this will free all TDcjButton objects
  // as well.
  if( FButtons ) delete FButtons;
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditDlg::DisplayButtons(void)
{
  // load the buttons info into the lstButtons list box

  // clear the listbox
  lstButtons->Clear();

  // make sure we have a valid pointer
  if( Buttons ) {
    // cycle through the buttons, adding them to the list box
    for( int i = 0; i < Buttons->Count; i++ )
      // add the button to the listbox and set the current index
      lstButtons->Items->AddObject(IntToStr(i) + " - " +
          Buttons->Items[i]->Caption, Buttons->Items[i]);
  }
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditDlg::cmdDeleteClick(TObject *Sender)
{
  // delete the selected TDcjButton from the collection and reload
  // the listbox.

  // make sure we have a valid selection
  if( lstButtons->ItemIndex != -1 ) {
    // delete the collection item from the collection
    Buttons->DeleteItem(dynamic_cast<TDcjButton *>
        (lstButtons->Items->Objects[iCurrentIndex]));
    // reload the list box
    DisplayButtons();
    // set no current index
    iCurrentIndex = -1;
    // clear the button properties
    ClearButtonProperties();
  }
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditDlg::DisplayButtonProperties(TDcjButton *pButton)
{
  // display the button's properties in the dialog
  txtCaption->Text = pButton->Caption;
  txtWidth->Text = IntToStr(pButton->Width);
  txtHeight->Text = IntToStr(pButton->Height);
  chkDefault->State = pButton->Default ? cbChecked : cbUnchecked;
  chkCancel->State = pButton->Cancel ? cbChecked : cbUnchecked;
  chkHelp->State = pButton->Help ? cbChecked : cbUnchecked;
  cboModalResult->ItemIndex = pButton->ModalResult;
  txtID->Text = IntToStr(pButton->ButtonID);
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditDlg::lstButtonsClick(TObject *Sender)
{
  // make sure we have a proper index
  if( iCurrentIndex != -1 )
    // store the properties for the item we just left
    StoreProperties(
        dynamic_cast<TDcjButton *>(lstButtons->Items->Objects[iCurrentIndex]));

  // set the CurrentIndex to the listbox index
  iCurrentIndex = lstButtons->ItemIndex;

  // make sure we have a valid selection
  if( iCurrentIndex != -1 ) {
    // display the properties
    DisplayButtonProperties(
        dynamic_cast<TDcjButton *>(lstButtons->Items->Objects[iCurrentIndex]));
    // set focus to the caption field
    txtCaption->SetFocus();
  }
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditDlg::txtCaptionChange(TObject *Sender)
{
  // make sure we have a valid index
  if( iCurrentIndex != -1 ) {
    // make sure the caption is not already set
    if( lstButtons->Items->Strings[iCurrentIndex] != (IntToStr(iCurrentIndex) +
        " - " + txtCaption->Text) ) {
      lstButtons->Items->Strings[iCurrentIndex] =
          IntToStr(iCurrentIndex) + " - " + txtCaption->Text;
      // needed to set focus back to the listbox
      lstButtons->ItemIndex = iCurrentIndex;
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditDlg::StoreProperties(TDcjButton *pButton)
{
  // store the properties in the TDcjButton
  pButton->Caption = txtCaption->Text;
  pButton->Width = StrToIntDef(txtWidth->Text, DEFAULT_BTN_WIDTH);
  pButton->Height = StrToIntDef(txtHeight->Text, DEFAULT_BTN_HEIGHT);
  pButton->Default = chkDefault->State == cbChecked;
  pButton->Cancel = chkCancel->State == cbChecked;
  pButton->Help = chkHelp->State == cbChecked;
  pButton->ModalResult = cboModalResult->ItemIndex;
  pButton->ButtonID = StrToIntDef(txtID->Text, 0);
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditDlg::ClearButtonProperties(void)
{
  // clear all of the fields that hold the button properties
  txtCaption->Text = "";
  txtWidth->Text = "";
  txtHeight->Text = "";
  chkDefault->State = cbUnchecked;
  chkCancel->State = cbUnchecked;
  chkHelp->State = cbUnchecked;
  cboModalResult->Text = -1;
  txtID->Text = "";
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditDlg::cmdOkClick(TObject *Sender)
{
  // store any changes made
  if( iCurrentIndex != -1 )
    // store the properties for the item we just left
    StoreProperties(
        dynamic_cast<TDcjButton *>(lstButtons->Items->Objects[iCurrentIndex]));
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditDlg::mnuPopupClick(TObject *Sender)
{
  // store any changes made
  if( iCurrentIndex != -1 )
    // store the properties for the item we just left
    StoreProperties(
        dynamic_cast<TDcjButton *>(lstButtons->Items->Objects[iCurrentIndex]));

  // add a new TDcjButton
  TDcjButton *button = FButtons->Add();

  // cast the sender object into a TMenuItem type
  TMenuItem *menu = dynamic_cast<TMenuItem *>(Sender);

  // local variables to store button properties
  AnsiString buttoncaption = menu->Caption;
  bool bdefault = false;
  bool bcancel = false;
  bool bhelp = false;
  TModalResult mresult = DEFAULT_BTN_MODALRESULT;

  // check which button template was selected
  switch( menu->Tag )  {
    case btDefault:
      buttoncaption = "(button)";
      break;
    case btOk:
      bdefault = true;
      mresult = mrOk;
      break;
    case btCancel:
      bcancel = true;
      mresult = mrCancel;
      break;
    case btHelp:
      bhelp = true;
      break;
    case btYes:
      mresult = mrYes;
      break;
    case btNo:
      mresult = mrNo;
      break;
    case btAbort:
      mresult = mrAbort;
      break;
    case btRetry:
      mresult = mrRetry;
      break;
    case btIgnore:
      mresult = mrIgnore;
  }

  // add the new button to the listbox and set the current index
  iCurrentIndex=lstButtons->Items->AddObject(IntToStr(lstButtons->Items->Count) +
      " - " + buttoncaption, button);

  // set the ButtonID to the current index
  button->ButtonID = iCurrentIndex;
  // set the button properties
  button->Caption = buttoncaption;
  button->Default = bdefault;
  button->Cancel = bcancel;
  button->Help = bhelp;
  button->ModalResult = mresult;

  // display the button information in the dialog
  DisplayButtonProperties(button);

  // set focus to the listbox
  lstButtons->ItemIndex = iCurrentIndex;

  // set focus to the caption field
  txtCaption->SetFocus();
}
//---------------------------------------------------------------------------
void __fastcall TDcjButtonsEditDlg::cmdNewMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  TPoint ScreenPoint, ClientPoint;
  // cast the Sender object to a TControl.  This way it will work with a
  // speed button or a command button, or any control derived from TControl
  // for that matter
  TControl *button = dynamic_cast<TControl *>(Sender);

  // convert the client coordinates to screen coordinates to place the menu
  // underneath the button.
  ClientPoint.x = 0;
  ClientPoint.y = button->Height;
  ScreenPoint = button->ClientToScreen(ClientPoint);
  // display the macros popup menu
  mnuPopup->Popup(ScreenPoint.x, ScreenPoint.y);
}
//---------------------------------------------------------------------------


